package ircbot;

import java.util.*;
import ircbot.modules.*;

/**
 * This class is a partial implementation of the IRCBotModule
 * interface. It provides the functionality to handle commands and
 * default implementations for several methods including
 * <code>parseMessage</code>, <code>randomThought</code>,
 * <code>getProbability</code>, <code>setModuleHandler</code>,
 * <code>getModuleHandler</code>, <code>loadState</code>,
 * <code>saveState</code>, and <code>compareTo</code>.
 */

public abstract class AbstractIRCBotModule implements IRCBotModule
{
    private ArrayList<String> publicCommands = new ArrayList<String>();
    private ArrayList<String> trusteeCommands = new ArrayList<String>();
    private ArrayList<String> chopCommands = new ArrayList<String>();
    private ArrayList<String> botopCommands = new ArrayList<String>();
    private Modules moduleHandler;

    public Collection<String> getCommands(Modules.CommandType type)
    {
        switch (type) {
        case PUBLIC: return publicCommands;
        case TRUSTEE: return trusteeCommands;
        case CHOP: return chopCommands;
        case BOTOP: return botopCommands;
        default: return null;
        }
    }

    /**
     * Declares a command that this module will handle. Adds the
     * command to the set of commands returned by getCommands.
     * @param cmnd The command to add, with prefix. E.g. "!seen".
     */
    protected void addCommand(Modules.CommandType type, String cmnd)
    {
        switch (type) {
        case PUBLIC: publicCommands.add(cmnd); break;
        case TRUSTEE: trusteeCommands.add(cmnd); break;
        case CHOP: chopCommands.add(cmnd); break;
        case BOTOP: botopCommands.add(cmnd); break;
        }
    }

    public String parseMessage(IRCMessage message)
    {
        return null;
    }

    public String randomThought(IRCChannel channel, Person sender)
    {
        return null;
    }

    public int getProbability()
    {
        return 0;
    }

    public String activateEvent(IRCChannel channel)
    {
        return null;
    }

    /**
     * Schedule an event to occur as soon as possible.
     */
    protected void scheduleEvent(IRCChannel channel)
    {
        scheduleEvent(channel, 0);
    }

    /**
     * Schedule an event some time in the future.
     * @param delay The minimum time that must pass before the event
     * is executed.
     */
    protected void scheduleEvent(IRCChannel channel, long delay)
    {
        IRCEvent event = new IRCEvent(channel, this, System.currentTimeMillis() + delay);
        getModuleHandler().getBot().scheduleEvent(event);
    }

    public void setModuleHandler(Modules handler)
    {
        moduleHandler = handler;
    }

    /**
     * The module handler can be used to find other modules loaded in
     * the bot. For instance to get hold of the People module to get a
     * Person object from a nick.
     * @return The module handler that was used to load this module.
     */
    protected Modules getModuleHandler()
    {
        return moduleHandler;
    }

    public int compareTo(IRCBotModule mod)
    {
        return Calc.rnd(0,mod.getProbability()) - Calc.rnd(getProbability());
    }

    public String toString()
    {
        return getModuleName();
    }

    public void loadState(SaveState state) { }
    public void saveState(SaveState state) { }
}
