package ircbot;

import ircbot.modules.*;
import java.io.*;
import java.util.*;

public class ChannelStats implements Serializable
{
    public static final long serialVersionUID = 2L;
    private static final int HOURS_TO_SAVE = 24 * 7;

    private long messCount;
    private double averageWord;
    private ArrayList<String> topicHistory;
    private int[] hourlyCount;
    private IRCMessage lastMessage = null; // The last thing sent to this channel by anyone.
    // TODO: More statistics for users and channels. Most active person.
    // TODO: HTTP server to show statistics.

    public ChannelStats()
    {
        topicHistory = new ArrayList<String>();
        hourlyCount = new int[HOURS_TO_SAVE];
        for (int i = 0; i < HOURS_TO_SAVE; i++)
            hourlyCount[i] = 0;
    }

    public long getMessageCounter() { return messCount; }
    public IRCMessage getLastMessage() { return lastMessage; }
    public Collection<String> getTopics() { return topicHistory; }

    public Collection<String> getTopics(int n)
    {
        ArrayList<String> topics = new ArrayList<String>();

        if (n > topicHistory.size())
            n = topicHistory.size();

        for (int i = 0; i < n; i++)
            topics.add(topicHistory.get(i));

        return topics;
    }

    public int getMessages(int hoursBack)
    {
        int now = (int)(System.currentTimeMillis() / Time.HOUR) %
            (HOURS_TO_SAVE);
        int count = 0;

        for (int i = 0; i < hoursBack; i++) {
            count += hourlyCount[now];
            now--;
            if (now < 0)
                now = HOURS_TO_SAVE - 1;
        }

        return count;
    }

    public void setTopic(String topic)
    {
        topicHistory.add(0, topic);
    }

    public void registerMessage(IRCMessage message)
    {
        averageWord = (averageWord * messCount + message.length()) / ++messCount;
        lastMessage = message;
        hourlyCount[(int)(System.currentTimeMillis() / Time.HOUR) %
                    HOURS_TO_SAVE]++;
    }
}
