package ircbot;

public abstract class IRCBotConfig
{
    public enum LoggingType { NONE, STANDARD, COLOR };
    LoggingType logging = LoggingType.NONE;
    boolean console = false;

    public IRCBotConfig(String[] args)
    {
        // TODO: Read bot name, channels to connect to, server etc from a config file (XML?)
        if (args == null)
            return;

        for (int i = 0; i < args.length; i++) {
            args[i] = args[i].toLowerCase();
            if (args[i].equals("debug")) {
                console = true;
            }
            else if (args[i].equals("log")) {
                logging = LoggingType.STANDARD;
            }
            else if (args[i].equals("color")) {
                logging = LoggingType.COLOR;
            }
            else if (args[i].equals("help")) {
                System.out.println("Command line options:");
                String[] options = commandLineOptions();
                for (int j = 0; j < options.length; j++)
                    System.out.println(options[j]);
                return;
            }
        }
    }

    public String[] commandLineOptions()
    {
        String[] op = {"debug - Turns on debug features. The most obvious one is that the bot \n" +
                       "connects to the console (stdin and stdout) instead of the real IRC \n" +
                       "network. Messages typed at the command line will be sent to the bot \n" +
                       "as IRC messages. \n",
                       "log - Enables logging. All traffic in the channel(s) will be piped \n" +
                       "to stdout. \n",
                       "color - Print log messages in color. Uses standard ANSI terminal \n" +
                       "codes to change color. \n",
                       "help - Display this help message and exit."};
        return op;
    }

    public LoggingType getLogging() { return logging; }
    public boolean getConsole() { return console; }

    public abstract String getIrcNetwork();
    public abstract String getBotName();
    public abstract String getBotNickPassword();
    public abstract String getBotInfoText();
    public abstract String getOperatorNick();
}
