package ircbot;

import ircbot.modules.*;

public class IRCMessage
{
    public static final long serialVersionUID = 0L;
    private String prefix;
    private String command;
    private String receiver;
    private String message;
    private String replyTo;
    private String botCommand;
    private String params;
    private String[] words;
    private Person sender;
    private IRCChannel channel;

    public IRCMessage(String msg, Channels channels, People people, String botName)
    {
        String nick;

        prefix = msg.substring(1, msg.indexOf(' '));
        msg = msg.substring(msg.indexOf(' ') + 1);
        command = msg.substring(0, msg.indexOf(' ')).toLowerCase();
        msg = msg.substring(msg.indexOf(' ') + 1);
        message = msg.substring(msg.indexOf(':') + 1).trim();

        if (msg.indexOf(':') > 0)
            receiver = msg.substring(0, msg.indexOf(':') - 1).toLowerCase();

        if (prefix.indexOf('!') != -1)
            nick = prefix.substring(0, prefix.indexOf('!'));
        else
            nick = prefix;

        replyTo = receiver;

        if (receiver != null && receiver.equals(botName.toLowerCase()))
            replyTo = nick;
        else if (command.equals("join"))
            replyTo = message;
        else if (command.equals("kick"))
            replyTo = receiver.substring(0, receiver.indexOf(' '));
        else if (command.equals("353"))
            replyTo = receiver.substring(receiver.indexOf('@') + 2);

        sender = people.getPerson(nick, true);
        channel = channels.getChannel(replyTo);

        if (command.equals("privmsg") && message.length() > 1 && IRCBot.isCommandPrefix(message.charAt(0))) {
            int endOfCommand = message.indexOf(' ');
            if (endOfCommand == -1)
                endOfCommand = message.length();
            else {
                params = message.substring(endOfCommand + 1).trim();
                words = params.split(" ");
            }
            botCommand = message.substring(0, endOfCommand).toLowerCase();
        }
    }

    public IRCMessage(IRCMessage oldMessage, String newCommand, String newParams)
    {
        prefix = oldMessage.prefix;
        command = newCommand;
        receiver = oldMessage.receiver;
        message = newCommand + " " + newParams;
        replyTo = oldMessage.replyTo;
        botCommand = newCommand;
        params = newParams;
        words = params.split(" ");
        sender = oldMessage.sender;
        channel = oldMessage.channel; 
    }

    public String getPrefix()      { return prefix; }
    public String getCommand()     { return command; }
    public String getReceiver()    { return receiver; }
    public String getMessage()     { return message; }
    public Person getSender()      { return sender; }
    public String getReplyTo()     { return replyTo; }
    public String getBotCommand()  { return botCommand; }
    public String getParams()      { return params; }
    public IRCChannel getChannel() { return channel; }

    public String getParam(int index)
    {
        if (words != null && words.length > index)
            return words[index];
        else
            return null;
    }

    /**
     * @return The length of this message. The length is defined as the number of words in the parameter list.
     * The length is only defined for bot commands. E.g. The command "!join channel password" has length 2.
     */
    public int length()
    {
        return words == null ? 0 : words.length;
    }

    public void replaceBotCommand(String command)
    {
        botCommand = command;
    }
}
