package ircbot;

import java.io.*;
import java.util.*;

public class PersonalStats implements Serializable
{
    public static final long serialVersionUID = 1L;

    private Date firstSeen  = null;
    private Date lastGood   = null;
    private Date lastJoin   = null;
    private Date lastLeave  = null;
    private Date lastActive = null;
    private long maxIdle    = 0;
    private long words      = 0;    /* Number of words sent to the channel. */
    private long messages   = 0;    /* Number of messages sent to the channel. */
    private long smileys    = 0;    /* Happy faces :) :-) :D :-D ;) ;-) :-] etc. */
    private long sadies     = 0;    /* Sad faces :( :-( :-[ etc. */
    private String quote    = null;
    private double capsLock = 0;    /* How much a person is shouting. */
    private ArrayList<Date> warnings = new ArrayList<Date>();

    public PersonalStats()
    {
        firstSeen = lastJoin = new Date();
    }

    public Date getFirstSeen()  { return firstSeen;  }
    public Date getLastGood()   { return lastGood;   }
    public Date getLastJoin()   { return lastJoin;   }
    public Date getLastLeave()  { return lastLeave;  }
    public Date getLastActive() { return lastActive; }
    public long getWords()      { return words;      }
    public long getMessages()   { return messages;   }
    public long getSmilies()    { return smileys;    }
    public long getSadies()     { return sadies;     }
    public String getQuote()    { return quote;      }
    public double getShoutFrequency()     { return capsLock; }
    public Collection<Date> getWarnings() { return warnings; }

    public long getMaxIdle()
    {
        if (lastActive != null) {
            if (System.currentTimeMillis() - lastActive.getTime() > maxIdle)
                maxIdle = System.currentTimeMillis() - lastActive.getTime();
        }
        else if (System.currentTimeMillis() - lastJoin.getTime() > maxIdle)
            maxIdle = System.currentTimeMillis() - lastJoin.getTime();
        return maxIdle;
    }

    public void setFirstSeen( Date time )  { firstSeen = time;  }
    public void setLastGood( Date time )   { lastGood = time;   }
    public void setLastJoin( Date time )   { lastJoin = time;   }
    public void setLastLeave( Date time )  { lastLeave = time;  }
    public void setLastActive( Date time ) { lastActive = time; }
    public void incMessages()    { messages++; }
    public void incSmileys()     { smileys++; }
    public void incSadies()      { sadies++; }
    public void addWords(long w) { words += w; }
    public void setQuote(String q) { quote = q; }

    public void registerShouting(double shout)
    {
        capsLock = (capsLock * (messages -1) + shout) / messages;
    }

    public void warn()
    {
        warnings.add(new Date());
    }

    public void clearWarnings()
    {
        warnings.clear();
    }

    public void mergeWith(PersonalStats ps)
    {
        if (firstSeen.getTime() > ps.getFirstSeen().getTime())
            firstSeen = ps.getFirstSeen();

        if (lastJoin.getTime() < ps.getLastJoin().getTime())
            lastJoin = ps.getLastJoin();

        if (ps.getLastActive() != null &&
                (lastActive == null || lastActive.getTime() < ps.getLastActive().getTime()))
            lastActive = ps.getLastActive();

        if (ps.getLastLeave() != null &&
                (lastLeave == null || lastLeave.getTime() < ps.getLastLeave().getTime()))
            lastLeave = ps.getLastLeave();

        if (ps.getLastGood() != null &&
                (lastGood == null || lastGood.getTime() < ps.getLastGood().getTime()))
            lastGood = ps.getLastGood();

        if (maxIdle < ps.getMaxIdle())
            maxIdle = ps.getMaxIdle();

        words += ps.getWords();
        messages += ps.getMessages();
        smileys += ps.getSmilies();
        sadies += ps.getSadies();
        capsLock = (capsLock + ps.getShoutFrequency()) / 2;
        
        if (quote == null)
            quote = ps.getQuote();

        warnings.addAll(ps.getWarnings());
    }
}
