package ircbot;

import java.util.*;
import java.io.*;

public class SaveState
{
    private HashMap<String, Object> state;
    private String module;

    @SuppressWarnings("unchecked")
    public SaveState(String name)
    {
        module = name;

        try {
            ObjectInputStream ois = new ObjectInputStream(new FileInputStream(name + ".state"));
            state = (HashMap<String, Object>)ois.readObject();
            ois.close();
        }
        catch (ClassNotFoundException e) {
            System.err.println("[loadState: " + e.getClass() + "] " + e.getMessage());
        }
        catch (IOException e) {
            System.err.println("[loadState: " + e.getClass() + "] " + e.getMessage());
        }
        if (state == null)
            state = new HashMap<String, Object>();
    }

    public void setModule(String moduleName)
    {
        module = moduleName;
    }

    /**
     * Fetch a data object from this state.
     * @param id The unique id that was used to store the data.
     * @return The data object associated to the given id or null if
     * no such id was found.
     */
    public Object get(String id)
    {
        return state.get(module + id);
    }

    /**
     * Store data into the state.
     * @param id A unique id that will identify the given data object
     * in this state. The id will be padded with the name of the
     * calling module to avoid name clashes between different modules.
     * @param data The data object to store.
     */
    public void add(String id, Object data)
    {
        state.put(module + id, data);
    }

    public void save(String filename)
    {
        try {
            ObjectOutputStream oos =
                new ObjectOutputStream(new FileOutputStream(filename));
            oos.writeObject(state);
            oos.close();
        }
        catch (IOException e) {
            System.err.println("[saveState: " + e.getClass() + "] " +
                               e.getMessage());
        }
    }
}
