package ircbot.modules;

import java.util.*;
import ircbot.*;

public class AI extends AbstractIRCBotModule
{
    private HashSet<String> goodWords;
    private HashSet<String> badWords;
    private HashSet<String> posWords = new HashSet<String>();
    private HashSet<String> negWords = new HashSet<String>();
    //private HashMap<String, WordState> vocabulary;

    public AI()
    {
        posWords.add("bra");
        posWords.add("rules");
        posWords.add("rulez");
        posWords.add("tillbe");
        posWords.add("hylla");

        negWords.add("suger");
        negWords.add("dålig");
        negWords.add("dåligt");
        negWords.add("dåliga");
        negWords.add("ful");
        negWords.add("dum");
        negWords.add("hata");
        negWords.add("hatar");
        negWords.add("bort");
        negWords.add("radera");
        negWords.add("förstöra");
        negWords.add("döda");
        negWords.add("begrava");

        addCommand(Modules.CommandType.PUBLIC, "!good");
        addCommand(Modules.CommandType.PUBLIC, "!bad");
    }

    public String getModuleName()
    {
        return "AI";
    }

    public String action(IRCMessage message)
    {
        if (message.getBotCommand().equals("!good")) {
            if (message.length() == 0 || !message.getSender().isTrustee()) {
                return "\001ACTION " + Language.select(message.getSender().getLanguage(),
                                    "tycker om: ", "Likes: ") + IRCBot.makeList(goodWords) + "\001";
            } else if (message.getParam(0).toLowerCase().equals("remove") && message.length() > 1) {
                String params = message.getParams().toLowerCase().substring(7);
                if (goodWords.contains(params.toLowerCase())) {
                    goodWords.remove(params.toLowerCase());
                }
                return params + " är inte ett fint ord.";
            } else if (goodWords.contains(message.getParams().toLowerCase())) {
                return Language.select(message.getSender().getLanguage(),
                                       "Jag tycker redan om ", "I already like ") + message.getParams() + ".";
            } else {
                if (badWords.contains(message.getParams().toLowerCase()))
                    badWords.remove(message.getParams().toLowerCase());
                goodWords.add(message.getParams().toLowerCase());
                return Language.select(message.getSender().getLanguage(),
                                       "Jag tycker numera om ", "Now, I do like ") + message.getParams() + ".";
            }
        }

        else if (message.getBotCommand().equals("!bad")) {
            if (message.length() == 0 || !message.getSender().isTrustee()) {
                return "\001ACTION " + Language.select(message.getSender().getLanguage(),
                                    "tycker inte om: ", "does not like: ") + IRCBot.makeList(badWords) + "\001";
            } else if (message.getParam(0).toLowerCase().equals("remove")) {
                String params = message.getParams().toLowerCase().substring(7);
                if (badWords.contains(params.toLowerCase())) {
                    badWords.remove(params.toLowerCase());
                }
                return params + " är inte ett fult ord.";
            } else if (badWords.contains(message.getParams().toLowerCase())) {
                return Language.select(message.getSender().getLanguage(),
                                       "Jag tycker redan illa om ", "I already dislike ") + message.getParams() + ".";
            } else {
                if (goodWords.contains(message.getParams().toLowerCase()))
                    goodWords.remove(message.getParams().toLowerCase());
                badWords.add(message.getParams().toLowerCase());
                return Language.select(message.getSender().getLanguage(),
                                       "Jag tycker numera inte om ", "Now, I don't like ") + message.getParams() + ".";
            }
        }

        return null;
    }

    public String parseMessage(IRCMessage message)
    {
        int feel = 0;
        int subj = 0;
        String lowMess = message.getMessage().toLowerCase();
        String mem = "[Säg till joppe att fel AI:108 har inträffat!]";
        StringTokenizer st = new StringTokenizer(lowMess, ".,:;()/?&-! \t\n\r\f");
        boolean requireAnswer = lowMess.contains("bottenanna");

        while (st.hasMoreTokens()) {
            String word = st.nextToken();
            if (posWords.contains(word)) feel++;
            else if (negWords.contains(word)) feel--;
            else if (goodWords.contains(word)) { subj++; mem = word; }
            else if (badWords.contains(word)) subj--;
        }

        if ((subj < 0 && feel >= 0) || (subj > 0 && feel < 0)) {
            message.getSender().notice(message.getChannel(), People.NoticeType.BAD);
            return IRCBot.maybeReply(0.7, "\001ACTION " + Language.select(message.getSender().getLanguage(),
                                          "tvättar ", "washes ") + message.getSender().getNick() +
                                          Language.select(message.getSender().getLanguage(),
                                          "s mun med tvål.\001", "'s mouth with soap.\001"));
        }
        else if (subj > 0 && feel >= 0) {
            message.getSender().notice(message.getChannel(), People.NoticeType.GOOD);
            return IRCBot.maybeReply(0.4, "\001ACTION " + Language.select(message.getSender().getLanguage(),
                                           "tycker om ", "likes ") + mem + "\001");
        }
        else if (subj < 0 && feel < 0) {
            message.getSender().notice(message.getChannel(), People.NoticeType.GOOD);
            return IRCBot.maybeReply(0.4, "\001ACTION " + Language.select(message.getSender().getLanguage(),
                                           "tycker om ", "likes ") + message.getSender().getNick() + "\001");
        }

        // TODO: AI should create a chat-session for each person who greets the bot. As long as the session is active messages from that person should get replies.
        if ((lowMess.contains("hej") ||
             lowMess.contains("hallå") ||
             lowMess.contains("tjena") ||
             lowMess.contains("tjoho")) &&
            lowMess.contains("bottenanna")) {
            return message.getSender().getNick() + ": Hej på dig du!";
        }

        if (lowMess.equals("bottenanna!")) {
            return message.getSender().getNick() + "!";
        }

        if (lowMess.contains("bäck") ||
            lowMess.contains("dike")) {
            /* ojasta allikkoon - från bäck till dike */
            return message.getSender().getNick() + ": ojasta allikkoon";
        }

        if (lowMess.contains("ojasta") ||
            lowMess.contains("allikkoon")) {
            /* ojasta allikkoon - från bäck till dike */
            return "\001ACTION vet allt om bäckar och diken...\001";
        }

        if (lowMess.contains("bottenanna") && !lowMess.startsWith("bottenanna")) {
            return IRCBot.maybeReply(0.3, message.getSender().getNick() + ": Bottenanna, det är jag det!");
        }

        if (requireAnswer) // TODO: This should be in Modules so that this reply gets lowest priority over other modules
            return message.getSender().getNick() + ": You talking to me?!";

        return null;
    }

    @SuppressWarnings("unchecked")
    public void loadState(SaveState state)
    {
        goodWords = (HashSet<String>)state.get("goodWords");
        badWords = (HashSet<String>)state.get("badWords");

        if (goodWords == null) goodWords = new HashSet<String>();
        if (badWords == null) badWords = new HashSet<String>();
    }

    public void saveState(SaveState state)
    {
        state.add("goodWords", goodWords);
        state.add("badWords", badWords);
    }

    public String getDescription(String topic, Language lang)
    {
        if (topic.equals(getModuleName().toLowerCase()))
            return Language.select(lang, "Modulen styr hur boten ska reagera på vanlig text som skrivs i kanalen.",
                    "The AI module controls how the bot should react on text sent to the channel.");
        else if (topic.equals("!good"))
            return "!good [remove] [word] --- " + Language.select(lang, "Visar vilka ord som är bättre än andra." +
            		" Om ett ord 'word' skickas med så läggs detta till i listan över fina ord. Om parametern 'remove'" +
            		" anges tas ordet bort.",
            		"Show words that are better than others. If a word is given, it will be added to the list of good" +
            		" words. If the parameter 'remove' is given in front of the word, it is removed.");
        else if (topic.equals("!bad"))
            return "!bad [remove] [word] --- " + Language.select(lang, "Visar ord som är sämre än andra. Om ett ord" +
            		" 'word' skickas med så läggs det till i listan över dåliga ord. Om parametern 'remove' anges" +
            		" tas ordet bort.",
            		"Show words that are considered bad. If a word is given it will be added to the list of bad words." +
            		" If the parameter 'remove' is given in front of the word, it is removed.");
        else
            return null;
    }

    public String getAuthor()
    {
        return "Jesper Wilhelmsson";
    }
}
