package ircbot.modules;

import java.util.*;
import ircbot.*;

public class Channels extends AbstractIRCBotModule
{
    private HashMap<String, IRCChannel> channels;

    public Channels()
    {
        addCommand(Modules.CommandType.PUBLIC, "!verbose");
        addCommand(Modules.CommandType.PUBLIC, "!settings");
        addCommand(Modules.CommandType.PUBLIC, "!messages");
        addCommand(Modules.CommandType.PUBLIC, "!topics");

        addCommand(Modules.CommandType.CHOP, "!greetings");
        addCommand(Modules.CommandType.CHOP, "!randomthought"); 
        addCommand(Modules.CommandType.CHOP, "!op");
        addCommand(Modules.CommandType.CHOP, "!halfop");
        addCommand(Modules.CommandType.CHOP, "!voice");

        addCommand(Modules.CommandType.BOTOP, "!channels");
        addCommand(Modules.CommandType.BOTOP, "!chremove");
        addCommand(Modules.CommandType.BOTOP, "!join");
        addCommand(Modules.CommandType.BOTOP, "!leave");
        addCommand(Modules.CommandType.BOTOP, "!quit");
    }

    public String getModuleName()
    {
        return "Channels";
    }

    public String action( IRCMessage message )
    {
        if ( message.getBotCommand().equals("!verbose") ) {
            IRCChannel channel = getChannel(message.getParam(0));
            int param = 0;

            if ( channel != null ) {
                param++;
                if (channel != message.getChannel() && !message.getSender().isOperator())
                    return "You are not privileged enough to do that.";
            } else {
                channel = message.getChannel();
            }

            if ( channel == null ) {
                return "Please specify a channel.";
            }

            if ( message.getParam(param) == null) {
                return "Verbose: " + (channel.isVerbose() ? "on" : "off");
            }

            if (message.getParam(param).toLowerCase().equals("on")) {
                message.getSender().notice(channel, People.NoticeType.GOOD);
                long silent = channel.setVerbose(true);
                if (silent < Time.MIN_5)
                    return Language.select(message.getSender().getLanguage(),
                            "Jag tänkte väl att det var ett misstag.",
                            "I knew that was made by misstake.");
                else if (silent < Time.MIN_30)
                    return Language.select(message.getSender().getLanguage(),
                            "Det var bättre!", "That's better!");
                else
                    return Language.select(message.getSender().getLanguage(),
                            "Äntligen!!", "Finally!");
            } else if (message.getParam(param).toLowerCase().equals("off")) {
                channel.setVerbose(false);
                message.getSender().notice(channel, People.NoticeType.BAD);
                return "Hrmpfh..!";
            }
        }

        else if (message.getBotCommand().equals("!messages")) {
            ChannelStats cs = message.getChannel().getStatistics();
            if (message.length() == 0) {
                return Language.select(message.getSender().getLanguage(),
                        "Det har skickats " + cs.getMessages(24 * 7) + " meddelanden den senaste veckan.",
                        cs.getMessages(24 * 7) + " messages was sent during the last week.");
            } else {
                int n = Calc.extractInt(message.getParams());
                if (n == -1)
                    return Language.select(message.getSender().getLanguage(),
                            "Användning: !messages [antal dagar]", "Usage: !messages [days]");
                if (n > 7)
                    n = 7;

                return Language.select(message.getSender().getLanguage(),
                        "Det har skickats " + cs.getMessages(24 * n) + " meddelanden de senaste " + n + " dagarna.",
                        cs.getMessages(24 * n) + " messages was sent during the last " + n + " days.");
            }
        }

        else if (message.getBotCommand().equals("!topics")) {
            ChannelStats cs = message.getChannel().getStatistics();
            if (message.length() == 0) {
                return IRCBot.makeList(cs.getTopics(5), false);
            } else {
                return IRCBot.makeList(cs.getTopics(Calc.extractInt(message.getParams())), false);
            }
        }

        else if (message.getBotCommand().equals("!channels")) {
            return IRCBot.makeList(channels.keySet());
        }

        else if (message.getBotCommand().equals("!chremove") && message.getParam(0) != null) {
            channels.remove(message.getParam(0));
            return IRCBot.makeList(channels.keySet());
        }

        else if (message.getBotCommand().equals("!greetings")) {
            IRCChannel channel = getChannel(message.getParam(0));
            int param = 0;

            if ( channel != null ) {
                param++;
                if (channel != message.getChannel() && !message.getSender().isOperator())
                    return "You are not privileged enough to do that.";
            } else {
                channel = message.getChannel();
            }

            if ( channel == null ) {
                return "Please specify a channel.";
            }

            if ( message.getParam(param) != null) {
                if (message.getParam(param).toLowerCase().equals("personal"))
                    channel.setGreetingType(IRCChannel.GreetingType.PERSONAL);
                else if (message.getParam(param).toLowerCase().equals("always"))
                    channel.setGreetingType(IRCChannel.GreetingType.ALWAYS);
                else if (message.getParam(param).toLowerCase().equals("none"))
                    channel.setGreetingType(IRCChannel.GreetingType.NONE);
            }
            return "Greetings: " + channel.getGreetingType();
        }

        else if (message.getBotCommand().equals("!randomthought")) {
            IRCChannel channel = getChannel(message.getParam(0));
            int param = 0;

            if ( channel != null ) {
                param++;
                if (channel != message.getChannel() && !message.getSender().isOperator())
                    return "You are not privileged enough to do that.";
            } else {
                channel = message.getChannel();
            }

            if ( channel == null ) {
                return "Please specify a channel.";
            }

            if ( message.getParam(param) != null) {
                if (message.getParam(param).toLowerCase().equals("on")) {
                    channel.setRandomMessage(true);
                } else if (message.getParam(param).toLowerCase().equals("off")) {
                    channel.setRandomMessage(false);
                }
            }
            return "Random thought: " + (channel.getRandomMessage() ? "on" : "off");
        }

        else if (message.getBotCommand().equals("!settings")) {
            IRCChannel channel;
            if (message.getParam(0) != null)
                channel = channels.get(message.getParam(0).toLowerCase());
            else
                channel = message.getChannel();

            if (channel != message.getChannel() && !message.getSender().isOperator())
                return "You are not privileged enough to do that.";

            if (channel != null) {
                return "Default language: " + channel.getLanguage() + "\n" +
                    "Verbose: " + (channel.isVerbose() ? "on" : "off") + "\n" +
                    "Greetings: " + channel.getGreetingType() + "\n" +
                    "Random thought: " + (channel.getRandomMessage() ? "on" : "off") + "\n";
            }
        }

        else if (message.getBotCommand().equals("!join") && message.getParam(0) != null) {
            IRCChannel channel = channels.get(message.getParam(0));
            if (channel == null) {
                if (message.getParam(1) != null)
                    channel = new IRCChannel(message.getParam(0), message.getParam(1));
                else
                    channel = new IRCChannel(message.getParam(0));
            }
            return "syscmd:" + join(channel);
        }

        else if (message.getBotCommand().equals("!leave") && message.length() > 0) {
            IRCChannel channel = channels.get(message.getParam(0));
            if (channel != null)
                return "syscmd:" + leave(channel);
        }

        else if (message.getBotCommand().equals("!op") ||
                 message.getBotCommand().equals("!halfop") ||
                 message.getBotCommand().equals("!voice")) {
            char mode = message.getBotCommand().charAt(1);
            IRCChannel target = getChannel(message.getParam(0));
            int firstNick = 0;
            boolean remove = false;

            if (target != null)
                firstNick++;
            else {
                target = message.getChannel();
                if (target == null)
                    return "As far as I'm concerned, this is not a channel.";
            }

            if (message.getParam(firstNick) == null) {
                switch (mode) {
                case 'o':
                    return "Channel operators: " + IRCBot.makeList(target.getOperators());
                case 'h':
                    return "Half operators: " + IRCBot.makeList(target.getHalfop());
                case 'v':
                    return "Voice: " + IRCBot.makeList(target.getVoice());
                }
            }

            if (message.getParam(firstNick).toLowerCase().equals("remove")) {
                remove = true;
                firstNick++;
            }

            String commandLine = "";
            while (message.getParam(firstNick) != null) {
                Person p = ((People)getModuleHandler().getModule("People")).getPerson(message.getParam(firstNick++));
                if (p != null) {
                    if (remove) {
                        switch (mode) {
                        case 'o': target.removeOperator(p); break;
                        case 'h': target.removeHalfOperator(p); break;
                        case 'v': target.removeVoice(p); break;
                        }
                        commandLine += "syscmd:mode " + target + " -" + mode + " " + p + "\n";
                    } else {
                        switch (mode) {
                        case 'o': target.addOperator(p); break;
                        case 'h': target.addHalfOperator(p); break;
                        case 'v': target.addVoice(p); break;
                        }
                        commandLine += "syscmd:mode " + target + " +" + mode + " " + p + "\n";
                    }
                }
            }
            return commandLine;
        }

        else if (message.getBotCommand().equals("!quit")) {
            return "syscmd:quit";
        }

        return null;
    }

    public String parseMessage( IRCMessage message )
    {
        if (message.getCommand().equals("privmsg")) {
            if (message.getChannel() != null)
                message.getChannel().getStatistics().registerMessage(message);
        }

        else if (message.getCommand().equals("topic")) {
            message.getChannel().getStatistics().setTopic(message.getMessage());
        }

        // 332 - Topic when joining a channel.
        else if (message.getCommand().equals("332")) {
            ChannelStats cs = message.getChannel().getStatistics();
            String topic = message.getMessage();
            if (cs.getTopics().size() > 0) {
                String lastTopic = cs.getTopics(1).toArray(new String[1])[0];
                if (!lastTopic.equals(topic)) {
                    cs.setTopic(topic);
                }
            }
            else
                cs.setTopic(topic);
        }

        else if (message.getCommand().equals("join")) {
            // TODO: Ideally NickServ should be contacted to see if the nick is identified before op:ing a person.
            message.getChannel().addUser(message.getSender());
            if (message.getChannel().isOperator(message.getSender()))
                return "syscmd:mode " + message.getChannel() + " +o " + message.getSender();

            if (message.getChannel().isHalfop(message.getSender()))
                return "syscmd:mode " + message.getChannel() + " +h " + message.getSender();

            if (message.getChannel().hasVoice(message.getSender()))
                return "syscmd:mode " + message.getChannel() + " +v " + message.getSender();
        }

        else if (message.getCommand().equals("quit")) {
            message.getChannel().removeUser(message.getSender());
        }

        else if (message.getCommand().equals("353")) {
            /* People already in the channel at arrival. */
            String[] names = message.getMessage().split(" ");
            People folks = (People)getModuleHandler().getModule("People");
            String reply = "";
            for (int i = 0; i < names.length; i++) {
                String nick = names[i];
                if ( (names[i].charAt(0) == '@') || (names[i].charAt(0) == '%') || (names[i].charAt(0) == '+') )
                    nick = names[i].substring(1);
                Person p = folks.getPerson(nick);
                if (p != null) {
                    message.getChannel().addUser(p);
                    if (names[i].charAt(0) == '@')
                        message.getChannel().addOperator(p);
                    else if (message.getChannel().isOperator(p))
                        reply += "syscmd:mode " + message.getChannel() + " +o " + nick + "\n";

                    if (names[i].charAt(0) == '%')
                        message.getChannel().addHalfOperator(p);
                    else if (message.getChannel().isHalfop(p))
                        reply += "syscmd:mode " + message.getChannel() + " +h " + nick + "\n";

                    if (names[i].charAt(0) == '+')
                        message.getChannel().addVoice(p);
                    else if (message.getChannel().hasVoice(p))
                        reply += "syscmd:mode " + message.getChannel() + " +v " + nick + "\n";
                }
            }
            return reply;
        }

        /* If we get kicked out of the channel, rejoin and give the
         * kicker what he or she deserves.
         */
        else if (message.getCommand().equals("kick")) {
            if (message.getReceiver().
                equals(message.getChannel().getName().toLowerCase() + " " +
                       getModuleHandler().getBot().getName().toLowerCase())) {
                int i = Calc.rnd(1000, 2000);
                message.getSender().notice(message.getChannel(), People.NoticeType.BAD);
                message.getSender().decScore(i);
                String rejoin = "syscmd:" + join(message.getChannel());
                String reply = "Hmpf.. Komma här och sparkas.. Det blir " + i + " minuspoäng!";
                String kick = "syscmd:KICK " + message.getReplyTo() + " " +
                    message.getSender().getNick() + " :War - never been so much fun...";
                return rejoin + '\n' + reply + '\n' + kick;
            }
        }

        return null;
    }

    public String randomThought(IRCChannel channel, Person sender)
    {
        ChannelStats cs = channel.getStatistics();

        return cs.getMessages(24 * 7) +
            Language.select(channel.getLanguage(),
                            " meddelanden skrevs i denna kanal under den senaste veckan.",
                            " messages was sent in this channel during the last week.");
    }

    public int getProbability()
    {
        return 60;
    }

    public String join(IRCChannel channel)
    {
        if (channels.get(channel.getName()) == null)
            channels.put(channel.getName(), channel);

        channel.channelActive();
        channel.setOnline(true);

        if (getModuleHandler().getModule("News") != null)
            getModuleHandler().getBot().scheduleEvent(new IRCEvent(channel,
                                           getModuleHandler().getModule("News"), 0));

        if (channel.getPassword() != null)
            return "join " + channel.getName() + " " + channel.getPassword();
        else
            return "join " + channel.getName();
    }

    public void joinOnline()
    {
        for (IRCChannel channel : channels.values())
            if (channel.isOnline())
                join(channel);
    }

    public String leave(IRCChannel channel)
    {
        getModuleHandler().getBot().unscheduleEvents(channel);
        channel.setOnline(false);
        return "part " + channel.getName();
    }

    public Collection<IRCChannel> getChannels()
    {
        return channels.values();
    }

    public IRCChannel getChannel(String name)
    {
        if (name == null)
            return null;
        return channels.get(name.toLowerCase());
    }

    @SuppressWarnings("unchecked")
    public void loadState(SaveState state)
    {
        channels = (HashMap<String, IRCChannel>)state.get("channels");
        if (channels == null)
            channels = new HashMap<String, IRCChannel>();
    }

    public void saveState(SaveState state)
    {
        state.add("channels", channels);
    }

    public String getDescription(String topic, Language lang)
    {
        if (topic.equals(getModuleName().toLowerCase()))
            return Language.select(lang, "Denna modul hanterar allt som rör kanalerna som boten besöker.",
                                   "This module handles anything related to the channels used by this bot.");
        else if (topic.equals("!verbose"))
            return "!verbose [channel] [on | off] --- " +
                Language.select(lang, "Bestämmer om boten får lägga sig i det som skrivs i kanalen. Endast den kanal " +
                                "som kommandot skrivs i påverkas.",
                                "Allows the bot to interfear with the discussion in the channel. Only the channel" +
                                "in which the command is entered is affected.");
        else if (topic.equals("!messages"))
            return "!msgstat [n] --- " +
                Language.select(lang, "Visar hur många meddelanden som skickats de senaste 'n' dagarna.",
                                "Display how many messages has been sent during the last 'n' days.");
        else if (topic.equals("!topics"))
            return "!topics [n] --- " +
                Language.select(lang, "Visar de senaste 'n' ämnena för kanalen, eller alla om inget 'n' anges.",
                                "Displays the 'n' last topics for the channel, or all if no 'n' is given.");
        else if (topic.equals("!quit"))
            return "!quit --- " + Language.select(lang, "Avsluta denna bot.", "Kill this bot.");
        else if (topic.equals("!channels"))
            return "!channels --- List all channels known to this bot.";
        else if (topic.equals("!chremove"))
            return "!chremove <channel> --- Remove the specified channel from this bot's list of channels.";
        else if (topic.equals("!settings"))
            return "!settings " + Language.select(lang, "[kanal] --- Visa kanalspecifika inställningar.",
                                "[channel] --- Show channel-specific settings.");
        else if (topic.equals("!greetings"))
            return "!greetings [channel] [on | off] --- " +
                Language.select(lang, "Bestämmer om boten ska hälsa på personer som kommer in i kanalen.",
                                "Determines if the bot should greet people who join the channel.");
        else if (topic.equals("!randomthought"))
            return "!randomthought [channel] [on | off] --- " +
                Language.select(lang, "Bestämmer om boten ska skicka genomtänkta kommentarer till kanalen om det varit tyst för länge.",
                                "Determines if the bot should send messages to the channel if it has been silent for too long.");
        else if (topic.equals("!join"))
            return "!join " + Language.select(lang, "<kanal> [lösenord] --- Låt boten gå in i den givna kanalen.",
                                "<channel> [password] --- Let the bot enter the given channel.");
        else if (topic.equals("!leave"))
            return "!leave " + Language.select(lang, "<kanal> --- Be boten lämna den givna kanalen.",
                                "<channel> --- Ask the bot to leave the given channel."); 
        else if (topic.equals("!op"))
            return "!op [remove] [nick1 [nick2 [...]]] --- Grant op privileges to 'nick'. Several nicks can be specified. Specify 'remove' to remove privileges.";
        else if (topic.equals("!halfop"))
            return "!halfop [remove] [nick1 [nick2 [...]]] --- Grant halfop privileges to 'nick'. Several nicks can be specified. Specify 'remove' to remove privileges.";
        else if (topic.equals("!voice"))
            return "!voice [remove] [nick1 [nick2 [...]]] --- Grant voice privileges to 'nick'. Several nicks can be specified. Specify 'remove' to remove privileges.";
        else
            return null;
    }

    public String getAuthor()
    {
        return "Jesper Wilhelmsson";
    }
}
