package ircbot.modules;

import java.util.*;
import ircbot.*;

public class Help extends AbstractIRCBotModule
{
    private HashMap<String, String> dictionary;

    public Help()
    {
        addCommand( Modules.CommandType.PUBLIC, "!help" );
    }

    public String getModuleName()
    {
        return "Help";
    }

    public String action( IRCMessage message )
    {
        if ( message.getSender().isTrustee() && message.length() > 0 ) {
            if ( message.getParam(0).equals("add") ) {
                String params = message.getParams().substring(4);
                if ( params.indexOf('=') != -1 ) {
                    String topic = params.substring( 0, params.indexOf('=') ).trim().toLowerCase();
                    String description = params.substring( params.indexOf('=') + 1 ).trim();
                    if ( topic.length() > 1 )
                        dictionary.put( topic, description );
                } else
                    return getDescription("!help", message.getSender().getLanguage());
                return null;
            } else if ( message.getParam(0).equals("remove") ) {
                dictionary.remove( message.getParams().substring(7).toLowerCase() );
                return null;
            }
        }

        String reply = help( message.getParams(), message );
        if (reply != null)
            return reply;
        else
            return "No such keyword: " + message.getParams();
    }

    public String parseMessage( IRCMessage message )
    {
        if ( message.getMessage().contains("vad är ") ) {
            int end = message.getMessage().length();
            if ( message.getMessage().endsWith("?") )
                end--;
            String topic = message.getMessage().substring( message.getMessage().indexOf("vad är ") + 7, end );
            if (topic.startsWith("en "))
                topic = topic.substring(3);
            else if (topic.startsWith("ett "))
                topic = topic.substring(4);
            return help( topic, message );
        } else if ( message.getMessage().endsWith("?") ) {
            String topic = message.getMessage().substring( 0, message.getMessage().length() - 1 ).trim();
            return help( topic, message );
        } else if (message.getMessage().contains(" är ")) {
            String topic = message.getMessage().substring(0, message.getMessage().indexOf(" är "));
            if (topic.startsWith("en "))
                topic = topic.substring(3);
            else if (topic.startsWith("ett "))
                topic = topic.substring(4);
            String desc = message.getMessage().substring(message.getMessage().indexOf(" är ") + 4);
            dictionary.put(topic, desc + ", enligt " + message.getSender().getNick());
        }
        return null;
    }

    private String help( String params, IRCMessage message )
    {
        if ( params == null ) {
            LinkedList<String> list = new LinkedList<String>(dictionary.keySet());
            for ( IRCBotModule mod : getModuleHandler().getModules() ) {
                list.add(mod.getModuleName().toLowerCase());
            }
            return Language.select( message.getSender().getLanguage(),
                                    "För hjälp om ett specifikt kommando eller en modul, skriv !help <nyckelord>. " +
                                    "Följande nyckelord finns: ",
                                    "To get help on a specific command or a module, type !help <keyword>. Available " +
                                    "keywords: " ) +
                IRCBot.makeList(list);
        }

        String topic = params.toLowerCase();
        if ( dictionary.containsKey(topic) )
            return "[" + topic + "] " + dictionary.get(topic);
        else {
            String reply = "";
            for (IRCBotModule mod : getModuleHandler().getModules()) {
                if (topic.equals(mod.getModuleName().toLowerCase())) {
                    String txt = "<" + mod.getModuleName() + "> " +
                        mod.getDescription(topic, message.getSender().getLanguage()) +
                        Language.select(message.getSender().getLanguage(),
                                        " Tillgängliga kommandon: ", " Available commands: ") +
                        IRCBot.makeList(mod.getCommands(Modules.CommandType.PUBLIC));
                    if (message.getSender().isOperator())
                        txt += Language.select(message.getSender().getLanguage(),
                                               " OpKommandon: ", " OpCommands: ") +
                            IRCBot.makeList(mod.getCommands(Modules.CommandType.BOTOP));
                    return txt + " Author: " + mod.getAuthor();
                }

                for (String cmnd : mod.getCommands(Modules.CommandType.PUBLIC)) {
                    if (topic.equals(cmnd) || topic.equals(cmnd.substring(1)))
                        reply += "[" + cmnd + "] " +
                            mod.getDescription(cmnd, message.getSender().getLanguage());
                }

                if (message.getSender().isOperator()) {
                    for (String cmnd : mod.getCommands(Modules.CommandType.BOTOP)) {
                        if (topic.equals(cmnd) || topic.equals(cmnd.substring(1)))
                            reply += "[" + cmnd + "] " +
                                mod.getDescription(cmnd, message.getSender().getLanguage());
                    }
                }
            }
            if (reply != "")
                return reply;
        }

        return null;
    }

    @SuppressWarnings("unchecked")
    public void loadState(SaveState state)
    {
        dictionary = (HashMap<String, String>)state.get("dictionary");
        if (dictionary == null) dictionary = new HashMap<String, String>();
    }

    public void saveState(SaveState state)
    {
        state.add("dictionary", dictionary);
    }

    public String getDescription(String topic, Language lang)
    {
        if (topic.equals(getModuleName().toLowerCase()))
            return Language.select(lang, "Denna modul hanterar ett uppslagsverk.",
                                   "This module handles a dictionary.");
        else if (topic.equals("!help"))
            return "!help [add | remove] [word [= description]] --- " +
                Language.select(lang,
                                "Visar hjälp om givet uppslagsord. För att lägga till nya uppslagsord används " +
                                "likhetstecken '=' mellan uppslagsord och beskrivning. Uppslagsorden kan innehålla " +
                                "mellanslag. För att radera uppslagsord skriver man 'remove' före ordet.",
                                "Provides help about the given topic. To add a new topic use the equal sign '=' " +
                                "between topic and description. Topics may contain spaces. To remove a topic type " +
                                "'remove' in front of it.");
        else
            return null;
    }

    public String getAuthor()
    {
        return "Jesper Wilhelmsson";
    }
}
