package ircbot.modules;

import ircbot.*;

public class Lang extends AbstractIRCBotModule
{
    public Lang()
    {
        addCommand( Modules.CommandType.PUBLIC, "!lang" );
        addCommand( Modules.CommandType.PUBLIC, "!rövare" );
    }

    public String getModuleName()
    {
        return "Lang";
    }

    public String action(IRCMessage message)
    {
        if (message.getBotCommand().equals("!lang")) {
            if ( message.getParams() == null ) {
                return message.getSender().getNick() + ": " + Language.select( message.getSender().getLanguage(),
                        "Din språkinställning: ", "Your language setting: " ) +
                        message.getSender().getLanguage();
            } else {
                if ( message.getParams().equals("sv") )
                    message.getSender().setLanguage(Language.SVENSKA);
                else if ( message.getParams().equals("en") )
                    message.getSender().setLanguage(Language.ENGLISH);
                else if ( message.getParams().equals("ro") )
                    message.getSender().setLanguage(Language.ROVARE);
            }
            return null;
        }

        else if (message.getBotCommand().equals("!rövare")) {
            return toRovare(message.getParams());
        }
        return null;
    }

    public String getDescription(String topic, Language lang)
    {
        if (topic.equals(getModuleName().toLowerCase()))
            return Language.select(lang, "Denna modul sköter om språkval för boten. " +
                    "Inställningarna är personliga. Språket på ett svar från boten avgörs av " +
                    "språkinställningen för den person som skriver något.",
                    "This module handles the bot's language settings. The settings are personal. The " +
                    "language on a reply from the bot depends on the setting for " +
                    "the person who writes something.");
        else
            return "!lang [sv | en | ro] --- " + Language.select(lang, "Väljer språk. Endast " +
                    "angivna förkortningar är giltiga. Om inget språk anges visas nuvarande inställningar.",
                    "Choose language. Only the given abrevations are valid. If no " +
                    "language is provided the current setting is displayed.");
    }

    public String getAuthor()
    {
        return "Jesper Wilhelmsson";
    }

    public static String toRovare(String s)
    {
        String result = "";
        for (char ch : s.toCharArray()) {
            if (new String("bcdfghjklmnpqrstvwxz").indexOf(ch) != 0)
                result += ch + "o" + ch;
            else if (new String("BCDFGHJKLMNPQRSTVWXZ").indexOf(ch) != 0)
                result += ch + "o" + Character.toLowerCase(ch);
            else
                result += ch;
        }
        return result;
    }
}
