package ircbot.modules;

import java.util.*;
import ircbot.*;

public class MasterMind extends AbstractIRCBotModule
{
    private HashMap<String, Game> games;

    public MasterMind()
    {
        games = new HashMap<String, Game>();
        addCommand( Modules.CommandType.PUBLIC, "!mm" );
    }

    public String getModuleName()
    {
        return "MasterMind";
    }

    public String action( IRCMessage message )
    {
        if (games.containsKey(message.getReplyTo()))
            return games.get(message.getReplyTo()).guess(message.getParams(),
                                                         message.getSender());
        else {
            Game g = new Game(message.getReplyTo(), message.getParams());
            games.put(message.getReplyTo(), g);
            return g.guess(null, message.getSender());
        }
    }

    public String getDescription(String topic, Language lang)
    {
        if (topic.equals(getModuleName().toLowerCase())) {
            return "Spelet går ut på att lista ut den rätta nyckeln. Detta " +
                "gör man genom att prova olika nycklar och kombinera ihop " +
                "svaren man får. Nyckeln består av ett antal siffror (0 - " +
                "9). För varje siffra i din nyckel får du tillbaka ett " +
                "svar; '-' betyder att siffran inte finns med i den rätta " +
                "nyckeln, '+' betyder att siffran finns med men ska vara " +
                "på en annan position, '*' betyder att siffran är rätt. " +
                "Notera att under vissa förutsättningar kan man få svaret " +
                "'-' även för siffror som finns med i nyckeln. Detta sker " +
                "då man har fler av samma siffra i sin nyckel och det " +
                "finns färre i den rätta nyckeln. Exempel: Lösning [4711] " +
                "- Prova nyckel [1118] - Svar [+-*-].";
        }
        else {
            return "!mm [level | key] --- Om spelet är aktivt provar man " +
                "nycklar genom att ange dem på kommandoraden, ex: " +
                "'!mm 1234'. Om spelet inte är aktivt startar man det med " +
                "'!mm'. Man kan även ange en svårighetsgrad mellan 1 och " +
                "10. Svårighetsgraden avgör längden på nyckeln.";
        }
    }

    public String getAuthor()
    {
        return "Jesper Wilhelmsson";
    }

    private String generateKey(int length)
    {
        String k = "";

        for (int i = 0; i < length; i++)
            k += Calc.rnd(0, 9);
        return k;
    }

    private class Game
    {
        private String owner;
        private ArrayList<String> moves;
        private String key;
        private int level;

        public Game(String owner, String params)
        {
            this.owner = owner;
            level = Calc.extractInt(params);
            if (level < 1 || level > 10)
                level = 4;
            key = generateKey(level);
            moves = new ArrayList<String>();
        }

        public String guess(String params, Person player)
        {
            if (params == null) {
                if (moves.size() > 0)
                    return "Ett spel pågår. Spelets historia: " +
                        IRCBot.makeList(moves, false);
                else {
                    return "Välkommen till Master mind! Försök lista ut " +
                        "nyckeln genom att skriva '!mm <nyckel>'. Nyckeln " +
                        "är " + level + " tecken lång.";
                }
            }
            else if (Calc.isNumber(params) &&
                     params.length() == level) {
                char[] newGuess = params.toCharArray();
                char[] keyCopy = key.toCharArray();

                for (int i = 0; i < level; i++) {
                    if (new String(keyCopy).indexOf(newGuess[i]) == -1)
                        newGuess[i] = '-';
                    else if (newGuess[i] == keyCopy[i]) {
                        newGuess[i] = '*';
                        keyCopy[i] = ' ';
                    }
                    else {
                        int pos = new String(keyCopy).indexOf(newGuess[i]);
                        if (newGuess[i] != newGuess[pos]) {
                            newGuess[i] = '+';
                            keyCopy[pos] = ' ';
                        }
                        else
                            newGuess[i] = '-';
                    }
                }

                String reply = "mm [" + params + "]  [" +
                    new String(newGuess) + "]  (" + player + ")";

                if (params.equals(key)) {
                    int score = 5 * level - moves.size();
                    if (score < 0)
                        score = 0;
                    reply += " Nyckeln hittad!";
                    player.incScore(score);
                    reply += " Det blir " + score + " poäng till " + player;
                    games.remove(owner);
                }

                moves.add(reply.substring(3));
                return reply;
            }
            else {
                return "Nyckeln jag tänker på innehåller endast siffrorna " +
                    "0 - 9 och är " + level + " tecken lång.";
            }
        }
    }
}
