package ircbot.modules;

import java.util.*;
import ircbot.*;

public class Numbers extends AbstractIRCBotModule
{
    private HashMap<String, Game> games;

    public Numbers()
    {
        games = new HashMap<String, Game>();
        addCommand( Modules.CommandType.PUBLIC, "!gissa");
        addCommand( Modules.CommandType.PUBLIC, "!numbers");
    }

    public String getModuleName()
    {
        return "Numbers";
    }

    public String action( IRCMessage message )
    {
        if ( games.containsKey(message.getReplyTo()) ) {
            String param;
            if (message.getBotCommand() != null)
                param = message.getParams();
            else
                param = message.getMessage();
            return games.get(message.getReplyTo()).guess(param, message.getSender());
        } else {
            Game g = new Game(message.getReplyTo(), message.getParams());
            games.put(message.getReplyTo(), g);
            return g.guess(null, message.getSender());
        }
    }

    public String parseMessage( IRCMessage message )
    {
        if ( games.containsKey(message.getReplyTo()) )
            if ( games.get(message.getReplyTo()).getLastActive() + Time.MIN_5 >
                 System.currentTimeMillis() && Calc.isNumber(message.getMessage()) ) {
                return action(message);
            }

        return null;
    }

    public String randomThought( IRCChannel channel, Person sender )
    {
        Game g = games.get(channel.getName());
        if (g == null) {
            g = new Game(channel.getName(), "1000");
            games.put(channel.getName(), g);
            return g.guess(null, sender);
        }
        else {
            return g.randomGuess(sender);
        }
    }

    public int getProbability()
    {
        return 100;
    }

    public String getDescription( String topic, Language lang )
    {
        if (topic.equals(getModuleName().toLowerCase())) {
            String sv = "Gissa talet. Jag tänker på ett tal mellan noll och en " +
                "övre gräns. Din uppgift är att gissa vilket tal det är. " +
                "Alla i kanalen kan vara med och gissa och det gäller att hinna " +
                "först till det rätta talet. Det finns poäng att hämta för den " +
                "som gissar rätt.";
            String en = "Guess the number. I'm thinking of a number between " +
                "zero and an upper limit. Your task is to guess what number " +
                "I'm thinking of. Everyone in the channel are welcome to " +
                "guess and there are points to gain for the one who finds " +
                "the answer first.";
            return Language.select(lang, sv, en);
        }
        else if (topic.equals("!gissa")) {
            return "!gissa [gräns | gissning] --- " +
                "Om spelet är aktivt gissar man genom att " +
                "skriva '!gissa <gissning>', om spelet inte är aktivt " +
                "skriver man '!gissa [övre gräns]' för att starta.";
        }
        else if (topic.equals("!numbers")) {
            return "!numbers [limit | guess] --- " +
                "If the game is active you guess by typing " +
                "'!guess <your guess>', otherwise you start " +
                "a new game by typing '!numbers [upper limit]'.";
        }
        else
            return null;
    }

    public String getAuthor()
    {
        return "Jesper Wilhelmsson";
    }

    private class Game
    {
        private String owner;
        private int number;
        private int guesses;
        private int guessMax;
        private int minSeen;
        private int maxSeen;
        private long lastActive;

        public Game(String owner, String params)
        {
            this.owner = owner;
            guessMax = Calc.extractInt(params);
            if (guessMax == -1)
                guessMax = 1000;
            number = Calc.rnd(0, guessMax);
            guesses = 0;
            minSeen = 0;
            maxSeen = guessMax;
            lastActive = System.currentTimeMillis();
        }

        public long getLastActive()
        {
            return lastActive;
        }

        public String guess(String params, Person player)
        {
            int g = Calc.extractInt(params);
            String reply;

            lastActive = System.currentTimeMillis();

            if (g == -1) {
                return Language.select(player.getLanguage(),
                                       "Jag tänker på ett tal mellan 0 och ",
                                       "I'm thinking of a number between 0 and ") +
                    guessMax;
            }

            guesses++;
            if (g == number) {
                int score = (int)(Math.log(guessMax) / Math.log(2)) * 2 - guesses;

                if (score < 1)
                    score = 1;

                player.incScore(score);

                reply = g + Language.select(player.getLanguage(), " var rätt!! Efter ", " was correct!! It took ") +
                    guesses + Language.select(player.getLanguage(), " gissningar. Det blir ", " guesses. That gives ") +
                    score + Language.select(player.getLanguage(), " poäng till ", " points to ") + player + ".";
                games.remove(owner);
            }
            else if (g > number) {
                reply = Language.select(player.getLanguage(),
                                        "Gissning " + guesses + ": " + g + " är för stort. Talet är mellan 0 och ",
                                        "Guess " + guesses + ": " + g + " is too large. The number is between 0 and ") +
                    guessMax + ".";
                if (g < maxSeen)
                    maxSeen = g;
            }
            else {
                reply = Language.select(player.getLanguage(),
                                        "Gissning " + guesses + ": " + g + " är för litet. Talet är mellan 0 och ",
                                        "Guess " + guesses + ": " + g + " is too small. The number is between 0 and ") +
                    guessMax + ".";
                if (g > minSeen)
                    minSeen = g;
            }

            return reply;
        }

        public String randomGuess(Person player)
        {
            lastActive = System.currentTimeMillis();
            return guess("" + Calc.rnd(minSeen + 1, maxSeen - 1), player);
        }
    }
}
