package ircbot.modules;

import java.util.*;
import ircbot.*;
import ircbot.modules.extras.*;

public class Pending extends AbstractIRCBotModule
{
    private ArrayList<Message> pending;

    public Pending()
    {
        addCommand( Modules.CommandType.PUBLIC, "!mess");
        addCommand( Modules.CommandType.CHOP, "!opmess");
        addCommand( Modules.CommandType.BOTOP, "!pending");
    }

    public String getModuleName()
    {
        return "Pending";
    }

    public String action( IRCMessage message )
    {
        String params = message.getParams();

        if (message.getBotCommand().equals("!mess")) {
            if (params == null) {
                ArrayList<String> msgs = new ArrayList<String>();
                Iterator<Message> it = pending.iterator();
                while (it.hasNext()) {
                    Message m = it.next();
                    if (m.getReceiver().equals(message.getSender())) {
                        msgs.add(m.getSender() + "; '" + m.getMessage() + "'");
                        it.remove();
                    }
                }
                return IRCBot.makeList(msgs);
            }
            else if (params.indexOf(' ') != -1) {
                String nick = params.substring(0, params.indexOf(' '));
                String mess = params.substring(params.indexOf(' ') + 1);
                pending.add(new Message(nick.toLowerCase(),
                                        message.getSender().getNick(), mess));
                return "Meddelandet kommer att levereras nästa gång " +
                    nick + " är aktiv.";
            }
        }

        if (message.getBotCommand().equals("!opmess") && params != null) {
            String nick = params.substring(0, params.indexOf(' '));
            String mess = params.substring(params.indexOf(' ') + 1);
            pending.add(new Message(nick.toLowerCase(),
                                    "Channel operator", mess));
            return "Meddelandet kommer att levereras nästa gång " +
                nick + " är aktiv.";
        }

        if (message.getBotCommand().equals("!pending")) {
            if (params != null && params.equals("clear")) {
                pending.clear();
            }
            else if (params != null && params.startsWith("clear ")) {
                String nick = params.substring(6);
                Iterator<Message> it = pending.iterator();
                while (it.hasNext()) {
                    Message m = it.next();
                    if (m.getReceiver().equals(nick)) {
                        it.remove();
                    }
                }
            }
            else {
                ArrayList<String> msgs = new ArrayList<String>();
                for (Message m : pending) {
                    if (params == null || m.getReceiver().equals(params)) {
                        msgs.add(m.getSender() + " -> " + m.getReceiver() +
                                 " [" + m.getMessage() + "]");
                    }
                }
                return IRCBot.makeList(msgs);
            }
        }

        return null;
    }

    public String parseMessage(IRCMessage message)
    {
        if ((message.getCommand().equals("privmsg")) ||
            (message.getCommand().equals("join"))) {
            Iterator<Message> it = pending.iterator();
            while (it.hasNext()) {
                Message m = it.next();
                if (m.getReceiver().equals(message.getSender().getNick().toLowerCase())) {
                    it.remove();
                    if (m.getSender().equals("bottenanna"))
                        return m.getReceiver() + ": " + m.getMessage();
                    else
                        return m.getReceiver() + ": Meddelande från " +
                            m.getSender() + " [" + m.getMessage() + "]";
                }
            }
        }

        return null;
    }

    public void saveState(SaveState state)
    {
        state.add("pending", pending);
    }

    @SuppressWarnings("unchecked")
    public void loadState(SaveState state)
    {
        pending = (ArrayList<Message>)state.get("pending");

        if (pending == null) pending = new ArrayList<Message>();
    }

    public String getDescription(String topic, Language lang)
    {
        if (topic.equals(getModuleName().toLowerCase()))
            return Language.select(lang, "Modul för att skicka meddelanden till personer som inte är aktiva i " +
                    "kanalen för tillfället.",
                    "A way to send messages to people not active in the channel right now.");
        else if (topic.equals("!mess"))
            return "!mess [nick message] --- " + Language.select(lang, "Skicka ett meddelande till 'nick'. " +
                    "Om inget argument anges visas eventuella väntande meddelanden.",
                    "Send a message to 'nick'. If no argument is given, your pending messages will be " +
                    "displayed.");
        else if (topic.equals("!opmess"))
            return "!opmess [nick message] --- " + Language.select(lang, "Skicka ett meddelande till 'nick' " +
                    "med boten som avsändare.",
                    "Send a message to 'nick' using the bot as sender.");
        else if (topic.equals("!pending"))
            return "!pending [clear] [nick] --- " + Language.select(lang, "Visa väntande meddelanden. Om 'nick' " +
                    "anges visas endast meddelanden till denne. 'clear' tar bort alla meddelanden " +
                    "alternativt alla till angivet 'nick'.",
                    "Display pending messages. If 'nick' is provided, only messages to 'nick' are " +
                    "displayed. The keyword 'clear' is used to remove all pending messages (or all messages " +
                    "addressed to the given 'nick').");
        else
            return null;
    }

    public String getAuthor()
    {
        return "Jesper Wilhelmsson";
    }
}
