package ircbot.modules;

import java.util.*;
import ircbot.*;

public class Quotes extends AbstractIRCBotModule
{
    private int index;
    private int replyUpperIndex;
    private long replyTimeout;
    private ArrayList<String> quotes = new ArrayList<String>();

    public Quotes()
    {
        index = 0;
        replyTimeout = System.currentTimeMillis();

        quotes.add("Once upon a time there was a pirate named Larry. Larry had his eye on a pirate princess named Cindy. Sadly Cindy's finger was already wearing the engagement ring of a bounder named Ted...");
        quotes.add("Once upon a time there was a pirate named Larry. A palm reader had told Larry that he was destined to marry a beautiful singer named Cindy. Tragically, Cindy had already pledged her heart to a jerk named Ted...");
        quotes.add("Once upon a time there was a pirate named Larry. Larry wanted to win the hand of a fair maiden named Cindy. Unfortunately, Cindy's hand had already been promised to a vile cad named Ted...");
        replyUpperIndex = quotes.size();
        quotes.add("The Gods await their new champion...");
        quotes.add("Prepare yourself, for the season of the Moonstone is upon you.");
        quotes.add("The full moon glares down ominously upon the landscape...");

        addCommand(Modules.CommandType.PUBLIC, "!quote");
    }

    public String getModuleName()
    {
        return "Quote";
    }

    public String action( IRCMessage message )
    {
        if (message.getParams() == null)
            return nextQuote(false);

        boolean showIndex = false;
        int n = Calc.extractInt(message.getParams());

        if (message.getParams().contains("index"))
            showIndex = true;

        if (message.getParams().contains("random"))
            return randomQuote(showIndex);
        else if (n != -1)
            return nthQuote(showIndex, n);
        else
            return nextQuote(showIndex);
    }

    public String parseMessage( IRCMessage message )
    {
        if (System.currentTimeMillis() < replyTimeout) {
            String mess = message.getMessage().toLowerCase();

            if (mess.contains("what?") ||
                mess.contains("what happened") ||
                mess.contains("what then") ||
                mess.contains("vad?") ||
                mess.equals("och?") ||
                mess.equals("sen då?") ||
                mess.startsWith("vad hände") ||
                mess.contains("vad kom sen")) {
                int ending = Calc.rnd(1, 3);

                replyTimeout = System.currentTimeMillis();
                if (ending == 1)
                    return "They all died of liver disorders " +
                        "in their early thirties. You wouldn't believe how " +
                        "much grog those kids were drinking. The End.";
                else if (ending == 2)
                    return "Eventually she followed her " +
                        "heart and married a quantum physicist from Azusa. " +
                        "Who wants to marry a pirate anyway? The End.";
                else if (ending == 3)
                    return "She decided that she couldn't " +
                        "stomach the thought of being married to either "+
                        "of them, so she joined a convent. The End.";
            }
        }

        return null;
    }

    public int getProbability()
    {
        return 70;
    }

    private String nextQuote( boolean verbose )
    {
        String reply = quotes.get(index);

        if (verbose)
            reply = index + ": " + reply;

        if (index < replyUpperIndex)
            replyTimeout = System.currentTimeMillis() + Time.MIN_5;

        index++;
        if (index == quotes.size())
            index = 0;

        return reply;
    }

    private String nthQuote( boolean verbose, int n )
    {
        String reply;

        if (n >= quotes.size())
            n = quotes.size() - 1;

        reply = quotes.get(n);

        if (verbose)
            reply = n + ": " + reply;

        if (index < replyUpperIndex)
            replyTimeout = System.currentTimeMillis() + Time.MIN_5;

        return reply;
    }

    private String randomQuote( boolean verbose )
    {
        int rnd = Calc.rnd(quotes.size());
        String reply = quotes.get(rnd);

        if (verbose)
            reply = rnd + ": " + reply;

        if (index < replyUpperIndex)
            replyTimeout = System.currentTimeMillis() + Time.MIN_5;

        return reply;
    }

    public String getDescription( String topic, Language lang )
    {
        if (topic.equals("quote")) {
            return Language.select(lang, "Ger ett citat.", "Gives a quote.");
        }
        else {
            return "!quote [random | n] [index] --- " +
                Language.select(lang, "Om ordet 'random' skickas med väljs " +
                        "ett slumpmässigt citat ur databasen, annars plockas citaten i tur och ordning. Skickas " +
                        "ett tal (n) med så väljs citatet med detta index. Skickas ordet 'index' med så " +
                        "visas citatets index tillsammans med citatet.",
                        "If the parameter 'random' is provided a random quote from the database is selected, " +
                        "otherwise quotes are picked in order. A number (n) can be provided to show the quote " +
                        "with that index. If the parameter 'index' is provided the quote's index is displayed.");
        }
    }

    public String getAuthor()
    {
        return "Jesper Wilhelmsson, Tim Eriksson";
    }
}
