package ircbot.modules;

/**
 * This module was originally created by Peter Jönsson. Minor
 * adjustments has been made by Jesper Wilhelmsson.
 */

import java.util.*;
import ircbot.*;

public class StenSaxPase extends AbstractIRCBotModule
{
    private HashMap<String, Game> games;

    public StenSaxPase()
    {
        games = new HashMap<String, Game>();
        addCommand(Modules.CommandType.PUBLIC, "!sten");
        addCommand(Modules.CommandType.PUBLIC, "!sax");
        addCommand(Modules.CommandType.PUBLIC, "!påse");
    }

    public String getModuleName()
    {
        return "Sten-Sax-Påse";
    }

    public String action( IRCMessage message )
    {
        if (games.containsKey(message.getSender().getNick()))
            return games.get(message.getSender().getNick()).
                spel(message.getBotCommand(), message.getSender());
        else {
            // Startar nytt spel med spelarens val som sten, sax eller påse.
            Game g = new Game(message.getSender().getNick());
            games.put(message.getSender().getNick(), g);
            return g.spel(message.getBotCommand(), message.getSender());
        }
    }

    // Bestämmer om spelet ska fortsätta eller avslutas utifrån spelarens
    // svar på förfrågan om fortsatt spel.
    public String parseMessage( IRCMessage message )
    {
        if (message.getSender() == null)
            return null;

        Game userGame = games.get(message.getSender().getNick());
        if (userGame != null && userGame.isWaiting()) {
            if (userGame.getTimeout() < System.currentTimeMillis()) {
                games.remove(message.getSender().getNick());
            }
            else if (message.getMessage().equals("nej")) {
                games.remove(message.getSender().getNick());
                return message.getSender().getNick() +
                    ": Fegis!! Du vann med " +
                    userGame.getUserScore() + "-" +
                    userGame.getMyScore() + ".";
            }
            else if (message.getMessage().equals("ja")) {
                return userGame.goOn();
            }
        }
        return null;
    }

    public String getDescription( String topic, Language lang )
    {
        if (topic.equals(getModuleName().toLowerCase())) {
            return "Spelet Sten-Sax-Påse";
        }
        else if (topic.equals("!sten"))
            return "!sten --- Ett av alternativen i spelet Sten-Sax-Påse.";
        else if (topic.equals("!sax"))
            return "!sax --- Ett av alternativen i spelet Sten-Sax-Påse.";
        else if (topic.equals("!påse"))
            return "!påse --- Ett av alternativen i spelet Sten-Sax-Påse.";
        else
            return null;
    }

    public String getAuthor()
    {
        return "Peter Jönsson";
    }

    // Slumpar fram sten, sax eller påse och skriver i kanalen om spelaren
    // vann eller förlorade. Vinner spelaren får han/hon förfrågan om att
    // fortsätta spela, annars avslutas spelet.
    private class Game
    {
        private String owner;
        private int bestOf;
        private int userScore;
        private int myScore;
        private boolean waitForAnswer;
        private long timeout;

        public Game(String owner)
        {
            this.owner = owner;
            userScore = 0;
            myScore = 0;
            waitForAnswer = false;
            timeout = System.currentTimeMillis();
            bestOf = 1;
        }

        public int getUserScore()
        {
            return userScore;
        }

        public int getMyScore()
        {
            return myScore;
        }

        public long getTimeout()
        {
            return timeout;
        }

        public String goOn()
        {
            waitForAnswer = false;
            timeout = System.currentTimeMillis();
            return "Då spelar vi bäst av " + bestOf + "!";
        }

        public boolean isWaiting()
        {
            return waitForAnswer;
        }

        public String spel(String command, Person player)
        {
            int spelarVal;
            int number = Calc.rnd(1, 3);
            String mittVal;
            String reply = "";

            if (waitForAnswer) {
                waitForAnswer = false;
                timeout = System.currentTimeMillis();
                reply = "Jag tolkar det som att du vill fortsätta.\n";
            }

            if (command.equals("!sten"))
                spelarVal = 1;
            else if (command.equals("!sax"))
                spelarVal = 2;
            else
                spelarVal = 3;

            switch (number) {
               case 1: mittVal = "sten"; break;
               case 2: mittVal = "sax"; break;
               default: mittVal = "påse";
            }

            if (spelarVal == number) {
                reply += "Jag hade också " + mittVal + ", Oavgjort!!";
                if (userScore != 0)
                    reply += " Det står nu " + userScore + "-" + myScore + ".";
                return reply;
            }
            else if ((spelarVal + 1 == number) ||
                     (spelarVal == 3 && number == 1)) {
                userScore++;
                player.incScore();
                bestOf += 2;
                waitForAnswer = true;
                timeout = System.currentTimeMillis() + 30000;
                return reply + "Jag hade " + mittVal + ", Du vann!! " +
                    "Bäst av " + bestOf + "? (ja/nej)";
            }
            else {
                myScore++;
                if (userScore == 0) {
                    games.remove(owner);
                    return reply + "Jag hade "+ mittVal + ", Du förlorade!!";
                }
                else if (myScore > userScore) {
                    games.remove(owner);
                    return reply + "Jag hade " + mittVal +
                        ", Du förlorade med " + userScore + "-" + myScore;
                }
                else
                    return reply + "Jag hade "+ mittVal + ", Du förlorade!! " +
                        "Det står nu " + userScore + "-" + myScore + ".";
            }
        }
    }
}
