package ircbot.modules;

import java.util.*;
import java.text.*;
import ircbot.*;

public class Time extends AbstractIRCBotModule
{
    public static final long MINUTE = 60000;
    public static final long MIN_5 = MINUTE * 5;
    public static final long MIN_10 = MINUTE * 10;
    public static final long MIN_20 = MINUTE * 20;
    public static final long MIN_30 = MINUTE * 30;
    public static final long HOUR = MINUTE * 60;
    public static final long HOUR_2 = HOUR * 2;
    public static final long HOUR_3 = HOUR * 3;
    public static final long HOUR_24 = HOUR * 24;

    public Time()
    {
        addCommand(Modules.CommandType.PUBLIC, "!time");
    }

    public String getModuleName()
    {
        return "Time";
    }

    public String action( IRCMessage message )
    {
        return time();
    }

    public String parseMessage( IRCMessage message )
    {
        if (message.getCommand().equals("privmsg")) {
            if (message.getMessage().contains("klocka") &&
                message.getMessage().endsWith("?"))
                return action(message);
        }

        return null;
    }

    public String randomThought(IRCChannel channel, Person sender)
    {
        return time();
    }

    public int getProbability()
    {
        return 75;
    }

    public String getDescription(String topic, Language lang)
    {
        if (topic.equals("time")) {
            return Language.select(lang, "Talar om vad klockan är.",
                                   "Gives the time.");
        }
        else
            return "!time --- " +
                Language.select(lang, "Talar om vad klockan är.",
                                "Gives the time.");
    }

    public String getAuthor()
    {
        return "Jesper Wilhelmsson";
    }

    private String time()
    {
        Date now = new Date();

        return "Klockan är " +
            new SimpleDateFormat("H:mm").format(now) + ". " +
            timeComment(now, null);
    }

    public static String timeComment(Date d, String nick)
    {
        String comment = "";
        String greeting = "";

        if (d == null)
            return null;

        switch (getHour(d)) {
        case 0:
        case 1:
            greeting = "Hej " + nick + "!";
            comment = "Det börjar bli dags att sova...";
            break;
        case 2:
            greeting = nick + ": Hallå där nattuggla!";
            comment = "Du borde verkligen gå och lägga dig!";
            break;
        case 3:
        case 4:
            greeting = "Hej " + nick + "!";
            comment = "Vad gör du här mitt i natten?!";
            break;
        case 5:
            greeting = "Zzzz... Tyst " + nick + "! Jag försöker sova...";
            comment = "zzZZZzzz";
            break;
        case 6:
            greeting = "Gäsp..! God morgon " + nick + "..";
            comment = "Vad gör du här vid den här tiden?";
            break;
        case 7:
            greeting = "God morgon " + nick + "!";
            comment = "Varför är du vaken så här tidigt?!";
            break;
        case 8:
        case 9:
        case 10:
        case 11:
            greeting = "God förmiddag " + nick + "!";
            break;
        case 12:
            greeting = "God middag " + nick + "!";
            comment = "Lunchdags?";
            break;
        case 13:
        case 14:
        case 15:
        case 16:
        case 17:
            greeting = "God eftermiddag " + nick + "!";
            break;
        case 18:
        case 19:
        case 20:
        case 21:
        case 22:
            greeting = "God afton " + nick + "!";
            break;
        case 23:
            greeting = "God afton " + nick + "!";
            comment = "Spöktimmen närmar sig...";
            break;
        }

        comment = maybeReplace(comment);

        if (nick != null)
            comment = greeting + " " + comment;

        return comment;
    }

    static private String maybeReplace(String comment)
    {
        int slump = Calc.rnd(10);

        switch(slump) {
        case 1: return "Anohter visitor. Stay a while. Stay forever!";
        case 2: return "Vad säger vi om vädret då?";
        case 3: return "Sett någon bra film på sistonde?";
        default: return comment;
        }
    }

    public static int getHour(Date d)
    {
        if (d == null)
            return -1;
        else
            return Integer.parseInt(new SimpleDateFormat("H").format(d));
    }

    public static String date2str(Date d)
    {
        if (d == null)
            return null;
        else
            return new SimpleDateFormat("H:mm d/M yyyy").format(d);
    }

    public static String time2str(long milli)
    {
        int seconds = (int)(milli / 1000) % 60;
        int minutes = (int)(milli / 60000) % 60;
        int hours = (int)(milli / 3600000) % 24;
        int days = (int)(milli / 86400000);
        String time = "";

        if (days > 0)
            time = days + " days ";
        if (hours > 0)
            time += hours + " hours ";
        if (minutes > 0)
            time += minutes + " minutes ";
        return time + seconds + " seconds.";
    }
}
